from .Magician import MagicianApi
from .MagicBox import MagicBoxApi
from .MagicianGo import MagicianGoApi
from .Lite import LiteApi
from .M1 import M1Api

__all__ = ("MagicianApi", "MagicBoxApi", "MagicianGoApi", "LiteApi", "M1Api")
